-- InitTimers function, must be called in the OnInitEvent function
function InitTimers()
	nxa.AddCustomItem("Seconds", "Continuous seconds timer", nxa.access.All, nxa.type.Real, "/", "SBCTimers")
	nxa.AddCustomItem("Minutes", "Continuous minutes timer", nxa.access.All, nxa.type.Integer, "/", "SBCTimers")
	nxa.AddCustomItem("Hours", "Continuous hours timer", nxa.access.All, nxa.type.Integer, "/", "SBCTimers")
	nxa.AddCustomItem("Seconds", "Seconds counter", nxa.access.All, nxa.type.Real, "/", "SBCCounters")
	nxa.AddCustomItem("Minutes", "Minutes counter", nxa.access.All, nxa.type.Integer, "/", "SBCCounters")
	nxa.AddCustomItem("Hours", "Hours counter", nxa.access.All, nxa.type.Integer, "/", "SBCCounters")
end

-------------
-- Runners --
-------------

function CallGeneric(path, value, func)
	local current = nxa.GetValue(path, 0)
	if (0 == math.fmod(current, value))
	then
		func()
	end
	nxa.WriteValue(path, current + 1)
end

-- Run function every x seconds
function CallEveryXSeconds(seconds, func)
	local path = "SBCTimers/Seconds"
	CallGeneric(path, seconds, func)
end

-- Run function every x minutes
function CallEveryXMinutes(minutes, func)
	local path = "SBCTimers/Minutes"
	CallGeneric(path, minutes, func)
end

-- Run Function every x hours
function CallEveryXHours(hours, func)
	local path = "SBCTimers/Hours"
	CallGeneric(path, hours, func)
end

--------------
-- Counters --
--------------

function CountDownGeneric(path, func)
	local current = nxa.GetValue(path, -1)
	-- if the value is positive or 0 subtract 1
	-- if the value is negative, do nothing
	if (current >= 0)
	then
		current = current - 1
		nxa.WriteValue(path, current)
	end
	if (current == 0)
	then
		-- execute function
		func()
	end
end

function StartSecondsCounter(seconds)
	local secondsPath = "SBCCounters/Seconds"
	nxa.WriteValue(secondsPath, seconds)
end

function StartMinutesCounter(minutes)
	local minutesPath = "SBCCounters/Minutes"
	nxa.WriteValue(minutesPath, minutes)
end

function StartHoursCounter(hours)
	local hoursPath = "SBCCounters/Hours"
	nxa.WriteValue(hoursPath, hours)
end

-- Count down from the seconds path
function CountDownSeconds(func)
    local secondsPath = "SBCCounters/Seconds"
	CountDownGeneric(secondsPath, func)
end

-- Count down from the minutes path
function CountDownMinutes(func)
    local minutesPath = "SBCCounters/Minutes"
	CountDownGeneric(minutesPath, func)
end

-- Count down from the hours path
function CountDownHours(func)
    local hoursPath = "SBCCounters/Hours"
	CountDownGeneric(hoursPath, func)
end

---------------------------
-- Own personal counters --
---------------------------

function CountDownOwnGeneric(path, func)
	local current = nxa.GetValue(path, -1)
	-- if the value is positive or 0 subtract 1
	-- if the value is negative, do nothing
	if (current >= 0)
	then
		current = current - 1
		nxa.WriteValue(path, current)
	end
	if (current == 0)
	then
		-- execute function
		func()
	end
end

function StartOwnSecondsCounter(seconds, secondsPath)
	nxa.WriteValue(secondsPath, seconds)
end

function StartOwnMinutesCounter(minutes, minutesPath)
	nxa.WriteValue(minutesPath, minutes)
end

function StartOwnHoursCounter(hours, hoursPath)
	nxa.WriteValue(hoursPath, hours)
end

-- Count down from the seconds path
function CountDownOwnSeconds(func, secondsPath)
	CountDownOwnGeneric(secondsPath, func)
end

-- Count down from the minutes path
function CountDownOwnMinutes(func, minutesPath)
	CountDownOwnGeneric(minutesPath, func)
end

-- Count down from the hours path
function CountDownOwnHours(func, hoursPath)
	CountDownOwnGeneric(hoursPath, func)
end
